# -*- coding: utf-8 -*-
"""
Logging utility, 
"""
from cfg.config import Config
import logging.config
#logging.config.fileConfig ('config/logging.ini', disable_existing_loggers=True)
#logger = logging.getLogger()
#def getLogger(name):
#    return logging.getLogger(name)
import logging
logging.basicConfig(filename='app.log', level=logging.INFO, filemode='a', format='%(asctime)s - %(name)s - %(levelname)s - %(message)s', datefmt="%Y-%m-%d-%H:%M:%S")

def info(msg):
    logging.info (msg)
    
def error(msg, err = None):
    print (f"{msg} {err}")
    if err is None:
        logging.error (f"{msg}", exc_info=True )
    else:
        #if err is not None:
        #    raise err
        logging.error (f"{msg} - {err}" )
        

def debug(msg, msg1="", msg2="", msg3="", msg4="", msg5=""):
    """
    Writes the logging messages in to default logging config
    """
    if 1==1: #Config.DEBUG:
        if msg1 == "":
            print (f"{msg}")
            logging.debug (msg)

            #logging.debug(msg)
        else:
            print (f"{msg}, {msg1}, {msg2}, {msg3}, {msg4}, {msg5}")
    


import talib as ta
import pandas as pd
#import dbmysql as db
import numpy as np
#from SQLBuild import insert, update, connection
#from datetime import datetime #, timedelta
#import fileutil
class CalcData():
    def __init__(self, freq, symbol, dt=None, epoch=None):
        self.fCLOSE = "close"
        self.fOPEN = "open"
        self.fHIGH = "high"
        self.fLOW = "low"
        self.fVOLUME = "volume"
        self.freq = freq
        self.symbol = symbol
        #self.dt = dt
        #self.epoch = epoch
        #self.setTable()

    def CalculateEMA(self, df):
        #df["highlow"] =(df[self.fHIGH] + df[self.fLOW])/2
        #close_vals = df[self.fCLOSE].values.astype('float64')
        close_vals = np.array(df[self.fCLOSE].values.astype('float64'), dtype=float)
        #high_vals  = np.array(df[self.fHIGH].values.astype('float64'), dtype=float)
        #low_vals  = np.array(df[self.fHIGH].values.astype('float64'), dtype=float)
        
        upper = close_vals.max()
        lower = close_vals.min()
        new_close_vals = np.array([(i-lower)/(upper-lower)*100 for i in close_vals])
        '''
        upper = df.groupby('date')["close"].transform('max')
        lower = df.groupby('date')["close"].transform('min')
        new_close_vals = (close_vals - lower)/(upper-lower)*100
        '''
        df["RSI"] = ta.RSI(close_vals)

        #upper = close_vals * 1.0075
        #lower = close_vals * .9925
        #new_close_vals = np.array([(i-lower)/(upper-lower)*100 for i in close_vals])
        df['new_close']=new_close_vals
        new_ema = ta.EMA(new_close_vals, timeperiod=20).round(1)
        df["slope_ema20"] = ta.LINEARREG_ANGLE(new_ema, timeperiod=2).round(0)

        new_sma = ta.SMA(new_close_vals, timeperiod=20).round(1)
        df["slope_sma20"] = ta.LINEARREG_ANGLE(new_sma, timeperiod=2).round(0)

        new_ema = ta.EMA(new_close_vals, timeperiod=7).round(1)
        df["slope_ema7"] = ta.LINEARREG_ANGLE(new_ema, timeperiod=2).round(0)

        #df["ema5"] = ta.EMA(close_vals, timeperiod=7).round(1)
        df["ema7"] = ta.EMA(close_vals, timeperiod=7).round(0)
        df["ema20"] = ta.EMA(close_vals, timeperiod=20).round(0)
        df["ema50"] = ta.EMA(close_vals, timeperiod=50).round(0)
        df["ema100"] = ta.EMA(close_vals, timeperiod=100).round(0)
        df["ema200"] = ta.EMA(close_vals, timeperiod=200).round(0)
        df["ubb"], df["sma20"], df["lbb"] = ta.BBANDS(df[self.fCLOSE].values,matype=ta.MA_Type.SMA, timeperiod=20)
        df=df.round(1)        
        return df

    def CalculateCPR(self, df):
        prevdf = df
        df['PIVOT'] = (prevdf[self.fHIGH]+prevdf[self.fLOW]+prevdf[self.fCLOSE])/3
        df['BC'] = (prevdf[self.fHIGH]+prevdf[self.fLOW])/2
        df['TC'] = df['PIVOT']*2-df['BC']
        df['R1'] = df['PIVOT']*2-prevdf[self.fLOW]
        df['S1'] = df['PIVOT']*2-prevdf[self.fHIGH]
        df['R2'] = df['PIVOT']-df['S1']+df['R1']
        df['S2'] = df['PIVOT']+df['S1']-df['R1']
        df['R3'] = df['PIVOT']-df['S2']+df['R2']
        df['S3'] = df['PIVOT']+df['S2']-df['R2']
        df['R4'] = df['PIVOT']-df['S3']+df['R3']
        df['S4'] = df['PIVOT']+df['S3']-df['R3']
        return df


    def Calculate(self, df):
        n=14
        close = np.array(df[self.fCLOSE].values, dtype='f8')
        df["RSI"] = pd.Series(ta.RSI(close, n), index = df.index, name='RSI%s' % str(4))
        bb = ta.BBANDS(df[self.fCLOSE].values, matype=ta.MA_Type.SMA, timeperiod=20)
        df["BBUpper"] = bb[0]
        df["BBMiddle"] = bb[1]
        df["BBLower"] = bb[2]
        #df["BBUpper"] =round(df["BBUpper"],1)

        df["LONGEMA"] = ta.EMA(df[self.fCLOSE].values, timeperiod=200)
        df["MEDIUMEMA"] = ta.EMA(df[self.fCLOSE].values, timeperiod=100)
        df["SHORTEMA"] = ta.EMA(df[self.fCLOSE].values, timeperiod=50)
        df["EMA20"] = ta.EMA(df[self.fCLOSE].values, timeperiod=20)
        df["QUICKEMA"] = ta.EMA(df[self.fCLOSE].values, timeperiod=7)
        df["trendangle"] = ta.LINEARREG_ANGLE(df[self.fCLOSE].values, timeperiod=7)
        df['trend'] = df.apply(lambda x: 'U' if x['trendangle']>0 else 'D', axis=1)

        df.round({self.fOPEN:1, self.fVOLUME:0, "BBUpper":1, "BBMiddle":1, "BBLower":1, "SHORTEMA":1})

        df["CDL3BLACKCROWS"] = ta.CDL3BLACKCROWS(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLDARKCLOUDCOVER"] = ta.CDLDARKCLOUDCOVER(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE], 0)
        df["CDLDOJI"] = ta.CDLDOJI(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLDOJISTAR"] = ta.CDLDOJISTAR(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLDRAGONFLYDOJI"] = ta.CDLDRAGONFLYDOJI(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLENGULFING"] = ta.CDLENGULFING(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLEVENINGDOJISTAR"] = ta.CDLEVENINGDOJISTAR(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE], 0)
        df["CDLEVENINGSTAR"] = ta.CDLEVENINGSTAR(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE], 0)
        df["CDLGAPSIDESIDEWHITE"] = ta.CDLGAPSIDESIDEWHITE(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLGRAVESTONEDOJI"] = ta.CDLGAPSIDESIDEWHITE(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLHAMMER"] = ta.CDLHAMMER(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLHANGINGMAN"] = ta.CDLHANGINGMAN(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])

        df["CDLHARAMI"] = ta.CDLHARAMI(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLHARAMICROSS"] = ta.CDLHARAMICROSS(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLHIGHWAVE"] = ta.CDLHIGHWAVE(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLHIKKAKE"] = ta.CDLHIKKAKE(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLHIKKAKEMOD"] = ta.CDLHIKKAKEMOD(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLHOMINGPIGEON"] = ta.CDLHOMINGPIGEON(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLIDENTICAL3CROWS"] = ta.CDLIDENTICAL3CROWS(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLINNECK"] = ta.CDLINNECK(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLINVERTEDHAMMER"] = ta.CDLINVERTEDHAMMER(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLKICKING"] = ta.CDLKICKING(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLKICKINGBYLENGTH"] = ta.CDLKICKINGBYLENGTH(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLLADDERBOTTOM"] = ta.CDLLADDERBOTTOM(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLLONGLEGGEDDOJI"] = ta.CDLLONGLEGGEDDOJI(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLLONGLINE"] = ta.CDLLONGLINE(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLMATCHINGLOW"] = ta.CDLMATCHINGLOW(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])

        df["CDLMATHOLD"] = ta.CDLMATHOLD(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLMORNINGDOJISTAR"] = ta.CDLMORNINGDOJISTAR(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLMORNINGSTAR"] = ta.CDLMORNINGSTAR(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLONNECK"] = ta.CDLONNECK(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLPIERCING"] = ta.CDLPIERCING(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLRICKSHAWMAN"] = ta.CDLRICKSHAWMAN(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLRISEFALL3METHODS"] = ta.CDLRISEFALL3METHODS(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLSEPARATINGLINES"] = ta.CDLSEPARATINGLINES(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLSHOOTINGSTAR"] = ta.CDLSHOOTINGSTAR(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLSHORTLINE"] = ta.CDLSHORTLINE(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLSPINNINGTOP"] = ta.CDLSPINNINGTOP(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])


        df["CDLSTALLEDPATTERN"] = ta.CDLSTALLEDPATTERN(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLSTICKSANDWICH"] = ta.CDLSTICKSANDWICH(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLTAKURI"] = ta.CDLTAKURI(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLTASUKIGAP"] = ta.CDLTASUKIGAP(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLTHRUSTING"] = ta.CDLTHRUSTING(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLTRISTAR"] = ta.CDLTRISTAR(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLUNIQUE3RIVER"] = ta.CDLUNIQUE3RIVER(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLUPSIDEGAP2CROWS"] = ta.CDLUPSIDEGAP2CROWS(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["CDLXSIDEGAP3METHODS"] = ta.CDLXSIDEGAP3METHODS(df[self.fOPEN], df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])

        df["OBV"] = ta.OBV(df[self.fCLOSE], df[self.fVOLUME])
        df["AD"] = ta.AD(df[self.fHIGH], df[self.fLOW], df[self.fCLOSE], df[self.fVOLUME])
        df["ADOSC"] = ta.ADOSC(df[self.fHIGH], df[self.fLOW], df[self.fCLOSE], df[self.fVOLUME], 3, 10)
        df["ADOSC"] = ta.ADOSC(df[self.fHIGH], df[self.fLOW],df[self.fCLOSE], df[self.fVOLUME])


        df["ADX14"] = ta.ADX( df[self.fHIGH], df[self.fLOW], df[self.fCLOSE],14)
        df["ADXR14"] = ta.ADXR( df[self.fHIGH], df[self.fLOW], df[self.fCLOSE],14)
        df["APO_12_26"] = ta.APO(df[self.fCLOSE])
        df["AROON_14_Up"],df["AROON_14_Down"] = ta.AROON( df[self.fHIGH], df[self.fLOW])
        df["RSI_14"] = ta.RSI(df[self.fCLOSE])
        df["STOCH_k"], df["STOCH_d"] = ta.STOCH( df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["STOCHF_k"], df["STOCHF_d"] = ta.STOCHF( df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["STOCHRSI_k"],df["STOCHRSI_d"] = ta.STOCHRSI(df[self.fCLOSE])
        df["TRIX_30"] = ta.TRIX(df[self.fCLOSE], 20)
        df["ULTOSC"] = ta.ULTOSC( df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["WILLR_14"] = ta.WILLR( df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["DX_14"] = ta.DX( df[self.fHIGH], df[self.fLOW], df[self.fCLOSE])
        df["MACD"], df["MACD_Signal"], df["MACD_History"] = ta.MACD(df[self.fCLOSE])
        df["MFI"] = ta.MFI( df[self.fHIGH], df[self.fLOW], df[self.fCLOSE], df[self.fVOLUME])
        df["MOM"] = ta.MOM(df[self.fCLOSE])
        df["ROC_10"] = ta.ROC(df[self.fCLOSE])
        df["ROCP_10"] = ta.ROCP(df[self.fCLOSE])
        df["ROCR_10"] = ta.ROCR(df[self.fCLOSE])

        df = np.round(df, 2)
        return df

s="""
    def Insert(self):
        sql = 'insert into derived_1min({cols}) values({vals})'
        cols = 'Trend_1Min  , Bollinger_LB_1Min  , Bollinger_MB_1Min  , Bollinger_UB_1Min  , Short_EMA__1Min  , Medium_EMA_1Min  , Long_EMA_1Min  , Chart_Pattern1_1Min  , Chart_Pattern2_1Min  , PCR_Selected_Option  , IdealValue  , RiskPercentage  , RiskRewardRatio '
        vals = '{Trend_1Min}, {Bollinger_LB_1Min}, {Bollinger_MB_1Min}, {Bollinger_UB_1Min}, {Short_EMA__1Min}, {Medium_EMA_1Min}, {Long_EMA_1Min}, {Chart_Pattern1_1Min}, {Chart_Pattern2_1Min}, {PCR_Selected_Option}, {IdealValue}, {RiskPercentage}, {RiskRewardRatio}'
        vals = vals.format()
        sql = sql.format(cols=cols, vals=vals)
    def setTable(self ):
        if self.freq == 1:
            self.sourcetable = 'ohlc_1min'
            self.targettable = 'derived_1min'
            self.readSql = 'select * from(select ADATE, OPEN, HIGH, LOW, CLOSE, VOLUME, OI from OHLC_1min where epochid <={epoch} and SYMBOL='{symbol}' order by adate desc limit 30) as a order by adate'
        if self.freq == 5:
            self.sourcetable = 'ohlc_5min'
            self.targettable = 'derived_5min'
            self.readSql = 'select * from(select ADATE, OPEN, HIGH, LOW, CLOSE, VOLUME, OI from OHLC_5min where epochid <={epoch} and SYMBOL='{symbol}' order by adate desc limit 30) as a order by adate'
        if self.freq == 86400:
            self.sourcetable = 'eoddata'
            self.targettable = 'derived_eod'
            self.readSql = 'select * from(select ADATE, OPEN, HIGH, LOW, CLOSE, tottrdqty as VOLUME, 0 as OI from eoddata where Adate <='{dt}' and SYMBOL='{symbol}' order by adate desc limit 30) as a order by adate'
    
    def ReadData(self):
        readsql = self.readSql.format(dt=self.dt.strftime("%Y-%m-%d"), symbol=self.symbol, epoch=self.epoch)
        df = pd.read_sql_query(readsql, db.getConn())
        return df

    def test(self):
        df = self.ReadData()
        df = self.CalculateCPR(df)
        #df=self.Calculate(df)
        #row=df.iloc[-1]
        #prt = "Upper: {BBU}, Middle: {BBM}, Lower: {BBL}, RSI: {RSI}"
        return df.iloc[-1]

if __name__ == '__main__':
    calc = CalcData(86400, 'BANKNIFTY', datetime.today(), 0)
    lower = 300
    upper = 638
    a = [422, 421, 420, 431, 433]
    a = [(i-lower)/(upper-lower)*100 for i in a]
    ar = np.array(a)
    s = ta.LINEARREG_ANGLE(ar, timeperiod=2).round(0)
if __name__ == '__main__':
    filename = fileutil.getNamedTempFile('abc.csv')
    df=pd.read_csv(filename)
    new_close_vals  = np.array(df['close'].values.astype('float64'), dtype=float)
    upper=new_close_vals.max()
    lower=new_close_vals.min()
    new_close_vals  = np.array([(i-lower)/(upper-lower)*100 for i in new_close_vals])
    df['newclose'] =new_close_vals
    df['newema'] = ta.EMA(new_close_vals, timeperiod=20).round(1)
    df['newema101'] = ta.EMA(new_close_vals, timeperiod=101).round(1)
    #df['newslope'] =ta.LINEARREG_SLOPE(df['ema20'].values, timeperiod=2).round(0)*180/3.14
    df['newslope'] =ta.LINEARREG_ANGLE(df['newema'].values, timeperiod=2).round(0)
    df.to_csv(filename)

calc.test()
"""



from enum import Enum
class SLType(Enum):
    fixed = 'fixed'
    price_action = 'priceaction'
    indicator = 'indicator'
    

class RiskAppetite (Enum):
    safe = 'safe'
    extra_safe = 'extra_safe'
    medium = 'medium'
    medium_opportunistic = 'medium_opportunistic'
    aggressive = 'aggressive'
    very_aggressive = 'very_aggressive'



# -*- coding: utf-8 -*-
"""
Created on Sun Sep  5 12:18:03 2021

@author: roope
"""
from cfg.config import Config
import requests
from datetime import datetime
bot_token = '1989230682:AAH1QXpYlQzgcdrZZtcFPzbMZsWmpaRQL3U'
def send(bot_message, freqtype = 'daily' ):
    cfg = Config()
    if freqtype=='minutely':
        if cfg.TESTING:
            bot_chatIDs = ['-1001518732952'] #test
        else:
            bot_chatIDs = ['-1001580520152'] #prod
    else:
        if cfg.TESTING:
            bot_chatIDs = ['-1001526466436'] #test
        else:
            bot_chatIDs = ['-1001555478330'] #prod
    for chatid in bot_chatIDs:
        send_text = 'https://api.telegram.org/bot' + bot_token + '/sendMessage?chat_id=' + chatid + '&parse_mode=Markdown&text=' + bot_message
        response = requests.get(send_text)

    return response.json()

def order_update(order):
    buysell = 'Buy' if order.buysell == 'b' else 'Sell'
    msg = f'{buysell}:{order.symbol} Qty:{order.qty} Price:{order.price}'
    send(msg, freqtype="minutely")

def eod_report(d, count, grossprofit, turnover):
    brokerage = count * 20
    d1 = d.strftime('%d-%b-%y')
    taxes = round(turnover/1000,0)
    netprofit=grossprofit - taxes - brokerage
    msg = f'''_EOD Report_ for {d1}
Total orders: {count}
Gross Profit: {grossprofit}
brokerage: {brokerage} 
taxes+others: {taxes} (approx.)

Net Profit: *{netprofit}*'''
    send(msg, freqtype="minutely")
    send(msg, freqtype="daily")

if __name__=='__main__':    
    eod_report(datetime.today(),8,  1535, 28827.5)
    


# -*- coding: utf-8 -*-
import tempfile
import os
from pathlib import Path
def getTempfile(suffix=''):
    filename = tempfile.TemporaryFile(suffix=suffix).name
    Path(filename).touch()
    return filename

def getNamedTempFile(filename, touch=True):
    f = tempfile.gettempdir()
    f = os.path.join(f, filename)
    if touch:
        Path(f).touch()
    return f




# -*- coding: utf-8 -*-
from datetime import datetime
from CalculateTA import CalcData
#import copy

class ORDER_STATUS:
    OPEN = 'OPEN'
    COMPLETE = 'COMPLETE'
    CANCELLED= 'CANCELLED'
    REJECTED= 'REJECTED'
    TRIGGER_PENDING = 'TRIGGER PENDING'

class Order_pref:
    profit_perc = .03
    sl_perc = .02
    order_price_threshold=.0025
    extreme_sl_perc=.1

    def __init__(self, profit_perc, sl_perc, order_price_threshold, extreme_sl_perc):
        self.profit_perc = profit_perc
        self.sl_perc =sl_perc
        self.order_price_threshold=order_price_threshold
        self.extreme_sl_perc = extreme_sl_perc
    def eff_target_price(self, price, buysell):
        if buysell==BuySell.buy:
            ret= (1+self.profit_perc) * price+.5
        if buysell==BuySell.sell:
            ret= (1-self.profit_perc) * price-.5
        return round(ret,1)
    def eff_sl_price(self, price, buysell):
        if buysell==BuySell.buy:
            ret= (1-self.sl_perc) * price-.5
        if buysell==BuySell.sell:
            ret =(1+self.sl_perc) * price+.5
        return round(ret,1)
    def eff_sl_extreme(self, price, buysell):
        if buysell==BuySell.buy:
            ret= (1-self.extreme_sl_perc) * price-10
        if buysell==BuySell.sell:
            ret =(1+self.sl_extreme_sl_perc) * price+10
        return round(ret,1)
    def eff_order_price_threshold(self, price, buysell):
        if buysell==BuySell.buy:
            ret=(1+self.order_price_threshold) * price
        if buysell==BuySell.sell:
            ret=(1-self.order_price_threshold) * price
        return round(ret,1)
    def eff_extreme_sl_price(self, price, buysell):
        if buysell==BuySell.buy:
            ret= (1-self.extreme_sl_perc) * price
        if buysell==BuySell.sell:
            ret =(1+self.extreme_sl_perc) * price
        return round(ret,1)


class action:
    new_position='New'
    none='None'
    stop_loss = 'SL'
    
class Position:
    symbol=''
    token=0
    qty=0 #qty can be negative if shorted
    price=0.0
    buysell=''
    target_price=0.0
    sl_price=0.0
    sl_extreme=0.0
    orderid=0
    comments=''
    last_action = action.none
    last_action_time = None

    def update(self, order, order_pref, sl=0):
        self.symbol = order.symbol
        self.token=order.token
        self.buysell=order.buysell
        self.last_action_time = order.at_time
        if self.buysell==BuySell.buy:
            self.price = (self.qty*self.price + order.qty*order.price)/(self.qty + order.qty)
            self.qty=self.qty+order.qty
            self.target_price = order_pref.eff_target_price(order.price,BuySell.buy)
            self.sl_extreme = order_pref.eff_sl_extreme(order.price,BuySell.buy)
            if sl == 0:
                self.sl_price = order_pref.eff_sl_price(order.price,BuySell.buy)
            else:
                self.sl_price = sl
        elif order.buysell==BuySell.sell:
            self.qty=self.qty-order.qty
        if self.qty==0:
            self.price=0
            self.last_action=action.none
        elif self.qty > 0:
            self.last_action=action.new_position
            
        self.orderid=0


        """
                multiplier=1 if self.position.qty<0 else -1
                self.position
                self.position.target_price = self.position.price * (1 + self.profit_perc * multiplier)
                self.position.target_price = round(self.position.target_price, 1)
                self.position.sl_price = self.position.price * (1 - self.sl_perc * multiplier)
                self.position.sl_price = round(self.position.sl_price , 1)

        """

class Order :
    exchangeToken=0
    strategyid=0
    code = 0
    orderid=0
    symbol=''
    token=0
    buysell=''
    qty=0
    pending_qty=0
    price=0
    trigger_price=0
    at_time=None
    ordtype =''
    status=''

    def __init__(self, symbol, exchangeToken, buysell, qty, price, comments, at_time=None,  ordtype='', trigger_price=0, strategyid=0, code=0):
        self.init_time=datetime.now()
        self.symbol=symbol
        self.exchangeToken=exchangeToken
        self.token=exchangeToken
        self.buysell=buysell
        self.qty=qty
        self.price=price
        self.comments=comments
        self.trigger_price = trigger_price
        self.ordtype = ordtype
        self.strategyid=strategyid
        self.code = code
        if at_time is None:
            self.at_time=datetime.now()
        else:
            self.at_time=at_time
    def to_string(self):
        return f'symbol: {self.symbol}, token: {self.token}, qty: {self.qty}, code: {self.code}, b/s: {self.buysell}'

    @classmethod
    def fromjson(self, ordjson):
        "Initialize MyData from a file"
        ord = self(ordjson['symbol'], 1,2,3,4,5,6,7)
        return ord

class Trend:
    up='U'
    down = 'D'
    sideways='S'

class PositionStatus:
    isopen='O'
    isclosed='C'

class Reco:
    def __init__(self, reco):
        self.reco=reco
        s_code = reco['specific_code']
        s_price = reco['specific_price']
        self.buysell=reco['buysell']
        self.direction = reco['direction']
        self.probability = reco['probability']
        self.id = reco['id']
        self.strategy= '' #reco['specific_strategy']
        if s_code != None and s_code != '':
            self.code = int(s_code)
        else:
            self.code=0
        self.close = reco['close']

        if s_price != None and s_price != '':
            self.price = int(s_price )
        else:
            self.price=0
        self.probability=int(reco['probability'])

    def copy(self):
        return Reco(self.reco)



class BuySell:
    buy='b'
    sell= 's'

class TradeSignal:
    buysell=""
    probability=0 #in percentage, between 1 and 100
    def __init__(self, buysell, probability):
        self.buysell=buysell
        self.probability = probability

class common_params:
    cpr_resistances=[]
    cpr_supports=[]
    cpr_top=0
    cpr_bottom=0
    global_markets_trend=Trend.sideways
    pcr=0
    pcr_trend=Trend.sideways
    maxpain=0
    maxpain_trend=Trend.sideways
    levels_up=[]
    levels_down=[]

class BB:
    bottom=0
    top=0
    middle=0
    def __init__(self, bottom, middle, top):
        self.bottom=bottom
        self.middle=middle
        self.top = top

class Values:
    token=0
    trend={}
    last_price=0
    df_1=None
    df_5=None
    df_15=None
    #ema_1m = {}
    #ema_5m = {}
    #ema_15m = {}
    trend_ema20_1=Trend.sideways
    trend_ema20_5=Trend.sideways
    trend_ema20_15=Trend.sideways
    trend_sma20_1=Trend.sideways
    trend_ema7_1=Trend.sideways
    angle_ema7_1=0
    angle_ema20=0

    sma20_1= 0
    ema20_1 = 0
    ema20_5 = 0
    ohlcs=None
    def __init__(self):
        self.levels_up=[36240, 35490, 35580]
        self.levels_down=[36140, 35200, 35100]
    def calculate(self):
        c=CalcData(1, 'BANKNIFTY', '2021-02-13', 234729489)

        self.df_1=c.CalculateEMA(self.df_1)
        row=self.df_1.iloc[-1]
        #angle = row.slope_ema20_1
        self.ema20_1= row.ema20
        self.sma20_1= row.sma20
        self.ema7_1= row.ema7
        self.ema200_1 = row.ema200

        self.angle_ema7_1 = row.slope_ema7
        self.angle_ema20_1 = row.slope_ema20
        self.angle_sma20_1 = row.slope_sma20

        self.trend_ema7_1 = self.angletoTrend(row.slope_ema7)
        self.trend_ema20_1 = self.angletoTrend(row.slope_ema20)
        self.trend_sma20_1 = self.angletoTrend(row.slope_sma20)

        self.ubb_1 = row.ubb
        self.lbb_1 = row.lbb

        self.touched_ema200 =  self.ema200_1 in range (int(self.df_1.tail(5).high.min()), int(self.df_1.tail(5).high.max()))
        self.closer_to_mbb = True if (self.ubb_1 - self.ema7_1)/(self.ema7_1 - self.sma20_1)>2 else False

        self.df_5=c.CalculateEMA(self.df_5)
        row=self.df_5.iloc[-1]
        self.ema20_5= row.ema20
        self.trend_ema20_5 = self.angletoTrend( row.slope_ema20)

        self.df_15=c.CalculateEMA(self.df_15)
        row=self.df_5.iloc[-1]
        self.ema20_15= row.ema20
        self.trend_ema20_15 = self.angletoTrend( row.slope_ema20)

    def __str__(self):
        return "ema20_1: {} sma20_1: {} ema7_1: {} ema200_1: {} angle_ema7_1 : {} angle_ema20_1: {} angle_sma20_1: {} trend_ema7_1 : {} trend_ema20_1: {} trend_sma20_1: {} ubb_1 : {} lbb_1 : {} touched_ema200: {} closer_to_mbb: {} ema20_5: {} trend_ema20_5: {} ema20_15: {} trend_ema20_15: {}".format(self.ema20_1, self.sma20_1, self.ema7_1, self.ema200_1, self.angle_ema7_1 , self.angle_ema20_1, self.angle_sma20_1, self.trend_ema7_1 , self.trend_ema20_1, self.trend_sma20_1, self.ubb_1 , self.lbb_1 , self.touched_ema200, self.closer_to_mbb, self.ema20_5, self.trend_ema20_5, self.ema20_15, self.trend_ema20_15)

    def angletoTrend(self, angle):
        if angle<-7 :
            return Trend.down
        elif angle<7:
            return Trend.sideways
        else:
            return Trend.up

    def is_1min_up(self):
        if (self.trend_ema20_1 == Trend.up):
            return True
        else:
            return False

    def is_1min_down(self):
        if (self.trend_ema20_1 == Trend.down):
            return True
        else:
            return False

    def is_all_up(self):
        if (self.trend_ema20_1 == self.trend_ema20_5) & (self.trend_ema20_5 == self.trend_ema20_15)  & (self.trend_ema20_1 == Trend.up):
            return True
        else:
            return False

    def is_all_up_except1(self):
        if (self.trend_ema20_15 == self.trend_ema20_5 ) & (self.trend_ema20_15 == Trend.up):
            return True
        else:
            return False

    def is_all_up_except5(self):
        if (self.trend_ema20_1 == self.trend_ema20_15)  & (self.trend_ema20_1 == Trend.up):
            return True
        else:
            return False

    def is_all_down(self):
        if (self.trend_ema20_1 == self.trend_ema20_5) & (self.trend_ema20_5 == self.trend_ema20_15)  & (self.trend_ema20_1 == Trend.down):
            return True
        else:
            return False

    def is_all_down_except5(self):
        if (self.trend_ema20_1 == self.trend_ema20_15)  & (self.trend_ema20_1 == Trend.down):
            return True
        else:
            return False

    def is_all_down_except1(self):
        if (self.trend_ema20_15 == self.trend_ema20_5)  & (self.trend_ema20_15 == Trend.down):
            return True
        else:
            return False



# -*- coding: utf-8 -*-
from datetime import date, datetime, timedelta
import time

def addSeconds(dt, seconds):
    return dt + timedelta(seconds=seconds)
    
def getTime():
    atime = datetime.now()
    if atime.hour * 60 + atime.minute > 929:
        atime = datetime.strptime("15:29", "%H:%M")
    return atime

def getDatefromEpoch(epoch):
    if epoch > 10000000000: 
        epoch=epoch/1000
    return date.fromtimestamp(epoch)

def getTimeIdfromEpoch(epoch):
    if epoch > 10000000000: 
        epoch=epoch/1000
    structt = time.localtime(epoch)
    mins=structt.tm_hour*60 + structt.tm_min
    return mins - 9*60 - 14

def dateQry(dt):
    return "str_to_date('{str}', '%Y-%m-%d')".format(str=dt.strftime("%Y-%m-%d"))

def getLatestEpoch(freq):
    return time.time()//(60*freq)*(60*freq) - (60*freq)
def get_nextday_of_week(dt, weekday:int):
    """Get the next weekday mentioned, if you want next thursday, pass 4 as weekday
    """
    # returns the next day of the week. 
    # weekday 1=monday, 2=tuesday, 3=wednesday, 4=thursday, 5=friday, 6=saturday, 7=sunday
    # if dt is a friday, and weekday is 4, then it returns the next thursday
    # if dt is a friday, and weekday is 3, then it returns the next wednesday

    weekday -= 1 # 0=monday, 1=tuesday, 2=wednesday, 3=thursday, 4=friday, 5=saturday, 6=sunday

    days_ahead = weekday - dt.weekday()
    if days_ahead <= 0: # Target day already happened this week
        days_ahead += 7
    return dt + timedelta(days_ahead)

if __name__ == '__main__':
    # test
    dt = datetime(2023, 1, 1).date() #Sunday

    print(dt, get_nextday_of_week(dt, 4)) #Get next thursday




